@@ -60,7 +60,7 @@ int main(int argc, char* argv[])
 {
   // Defaults
   umask(S_IRWXG | S_IRWXO);
-  Util::SetApplicationDir(std::string(getenv("HOME")) + std::string("/.nmail"));
+  Util::SetApplicationDir(Util::GetDefaultApplicationDir());
   Log::SetVerboseLevel(Log::INFO_LEVEL);
   bool online = true;
   bool changePass = false;
@@ -149,7 +149,7 @@ int main(int argc, char* argv[])
   THREAD_REGISTER();
   Util::InitAppSignalHandlers();
 
-  const std::string appVersion = Version::GetUiAppVersion();
+  const std::string appVersion = Version::GetAppName(true /*p_WithVersion*/);
   LOG_INFO("%s", appVersion.c_str());
   std::string osArch = Util::GetOsArch();
   LOG_INFO("%s", osArch.c_str());
@@ -203,6 +203,7 @@ int main(int argc, char* argv[])
     { "downloads_dir", "" },
     { "idle_timeout", "29" },
     { "sni_enabled", "1" },
+    { "logdump_enabled", "0" },
   };
   const std::string mainConfigPath(Util::GetApplicationDir() + std::string("main.conf"));
   std::shared_ptr<Config> mainConfig = std::make_shared<Config>(mainConfigPath, defaultMainConfig);
@@ -212,7 +213,8 @@ int main(int argc, char* argv[])
   const bool isSetup = !setup.empty();
   if (isSetup)
   {
-    if ((setup != "gmail") && (setup != "gmail-oauth2") && (setup != "icloud") && (setup != "outlook") && (setup != "outlook-oauth2"))
+    if ((setup != "gmail") && (setup != "gmail-oauth2") && (setup != "icloud") && (setup != "outlook") &&
+        (setup != "outlook-oauth2"))
     {
       std::cerr << "error: unsupported email service \"" << setup << "\".\n\n";
       ShowHelp();
@@ -285,6 +287,7 @@ int main(int argc, char* argv[])
   Util::SetDownloadsDir(mainConfig->Get("downloads_dir"));
   const bool isCoredumpEnabled = (mainConfig->Get("coredump_enabled") == "1");
   const bool sniEnabled = (mainConfig->Get("sni_enabled") == "1");
+  const bool isLogdumpEnabled = (mainConfig->Get("logdump_enabled") == "1");
 
   // Set logging verbosity level based on config, if not specified with command line arguments
   if (Log::GetVerboseLevel() == Log::INFO_LEVEL)
@@ -360,7 +363,8 @@ int main(int argc, char* argv[])
   }
   else
   {
-    if (!ObtainCacheEncryptPassword(isSetup, user, setupAllowCacheEncrypt, pass, smtpUser, smtpPass, secretConfig, mainConfig))
+    if (!ObtainCacheEncryptPassword(isSetup, user, setupAllowCacheEncrypt, pass, smtpUser, smtpPass, secretConfig,
+                                    mainConfig))
     {
       return 1;
     }
@@ -388,7 +392,7 @@ int main(int argc, char* argv[])
 
   Auth::Init(auth, authEncrypt, pass, isSetup);
 
-  Ui ui(inbox, address, name, prefetchLevel, prefetchAllHeaders);
+  std::shared_ptr<Ui> ui = std::make_shared<Ui>(inbox, address, name, prefetchLevel, prefetchAllHeaders);
 
   std::shared_ptr<ImapManager> imapManager =
     std::make_shared<ImapManager>(user, pass, imapHost, imapPort, online,
@@ -397,37 +401,37 @@ int main(int argc, char* argv[])
                                   idleTimeout,
                                   foldersExclude,
                                   sniEnabled,
-                                  std::bind(&Ui::ResponseHandler, std::ref(ui), std::placeholders::_1,
+                                  std::bind(&Ui::ResponseHandler, ui.get(), std::placeholders::_1,
                                             std::placeholders::_2),
-                                  std::bind(&Ui::ResultHandler, std::ref(ui), std::placeholders::_1,
+                                  std::bind(&Ui::ResultHandler, ui.get(), std::placeholders::_1,
                                             std::placeholders::_2),
-                                  std::bind(&Ui::StatusHandler, std::ref(ui), std::placeholders::_1),
-                                  std::bind(&Ui::SearchHandler, std::ref(ui), std::placeholders::_1,
+                                  std::bind(&Ui::StatusHandler, ui.get(), std::placeholders::_1),
+                                  std::bind(&Ui::SearchHandler, ui.get(), std::placeholders::_1,
                                             std::placeholders::_2),
                                   idleInbox, inbox);
 
   std::shared_ptr<SmtpManager> smtpManager =
     std::make_shared<SmtpManager>(smtpUser, smtpPass, smtpHost, smtpPort, name, address, online,
                                   networkTimeout,
-                                  std::bind(&Ui::SmtpResultHandler, std::ref(ui), std::placeholders::_1),
-                                  std::bind(&Ui::StatusHandler, std::ref(ui), std::placeholders::_1));
+                                  std::bind(&Ui::SmtpResultHandler, ui.get(), std::placeholders::_1),
+                                  std::bind(&Ui::StatusHandler, ui.get(), std::placeholders::_1));
 
   OfflineQueue::Init(queueEncrypt, pass);
 
-  ui.SetImapManager(imapManager);
-  ui.SetTrashFolder(trash);
-  ui.SetDraftsFolder(drafts);
-  ui.SetSentFolder(sent);
-  ui.SetClientStoreSent(clientStoreSent);
-  ui.SetSmtpManager(smtpManager);
+  ui->SetImapManager(imapManager);
+  ui->SetTrashFolder(trash);
+  ui->SetDraftsFolder(drafts);
+  ui->SetSentFolder(sent);
+  ui->SetClientStoreSent(clientStoreSent);
+  ui->SetSmtpManager(smtpManager);
 
   imapManager->Start();
   smtpManager->Start();
 
-  ui.Run();
+  ui->Run();
 
-  ui.ResetSmtpManager();
-  ui.ResetImapManager();
+  ui->ResetSmtpManager();
+  ui->ResetImapManager();
 
   smtpManager.reset();
   imapManager.reset();
@@ -446,9 +450,10 @@ int main(int argc, char* argv[])
 
   Util::CleanupStdErrRedirect();
 
+  ui.reset();
   LOG_INFO("exit");
 
-  Log::Cleanup();
+  Log::Cleanup(isLogdumpEnabled);
 
   return 0;
 }
@@ -462,47 +467,46 @@ static void ShowHelp()
     "Usage: nmail [OPTION]\n"
     "\n"
     "Options:\n"
-    "   -c, --cache-encrypt     prompt for cache encryption during oauth2 setup\n"
-    "   -d, --confdir <DIR>     use a different directory than ~/.nmail\n"
-    "   -e, --verbose           enable verbose logging\n"
-    "   -ee, --extra-verbose    enable extra verbose logging\n"
-    "   -h, --help              display this help and exit\n"
-    "   -k, --keydump           key code dump mode\n"
-    "   -o, --offline           run in offline mode\n"
-    "   -p, --pass              change password\n"
-    "   -s, --setup <SERVICE>   setup wizard for specified service, supported\n"
-    "                           services: gmail, gmail-oauth2, icloud, outlook,\n"
-    "                           outlook-oauth2\n"
-    "   -v, --version           output version information and exit\n"
-    "   -x, --export <DIR>      export cache to specified dir in Maildir format\n"
+    "   -c, --cache-encrypt        prompt for cache encryption during oauth2 setup\n"
+    "   -d, --confdir <DIR>        use a different directory than ~/.config/nmail\n"
+    "   -e, --verbose              enable verbose logging\n"
+    "   -ee, --extra-verbose       enable extra verbose logging\n"
+    "   -h, --help                 display this help and exit\n"
+    "   -k, --keydump              key code dump mode\n"
+    "   -o, --offline              run in offline mode\n"
+    "   -p, --pass                 change password\n"
+    "   -s, --setup <SERVICE>      setup wizard for specified service, supported\n"
+    "                              services: gmail, gmail-oauth2, icloud, outlook,\n"
+    "                              outlook-oauth2\n"
+    "   -v, --version              output version information and exit\n"
+    "   -x, --export <DIR>         export cache to specified dir in Maildir format\n"
     "\n"
     "Examples:\n"
-    "   nmail -s gmail          setup nmail for a gmail account\n"
-    "   nmail                   running nmail without setup wizard will generate\n"
-    "                           default configuration files in ~/.nmail dir if\n"
-    "                           not present already, these can be manually edited\n"
-    "                           to configure nmail for email service providers not\n"
-    "                           supported by the built-in setup wizard (refer to\n"
-    "                           FILES section on ~/.nmail/main.conf for details)\n"
+    "   nmail -s gmail             setup nmail for a gmail account\n"
+    "   nmail                      running nmail without setup wizard will generate\n"
+    "                              default configuration files in the nmail dir if\n"
+    "                              not present already, these can be edited to\n"
+    "                              configure nmail for email service providers not\n"
+    "                              supported by the built-in setup wizard (refer to\n"
+    "                              FILES section for details)\n"
     "\n"
     "Files:\n"
-    "   ~/.nmail/auth.conf      configures custom oauth2 client id and secret\n"
-    "   ~/.nmail/main.conf      configures mail account and general settings, for\n"
-    "                           full nmail functionality the following fields\n"
-    "                           shall be configured:\n"
-    "                           address (ex: example@example.com),\n"
-    "                           drafts (folder name, ex: Drafts),\n"
-    "                           imap_host (ex: imap.example.com),\n"
-    "                           imap_port (ex: 993),\n"
-    "                           inbox (folder name, ex: Inbox),\n"
-    "                           name (ex: Firstname Lastname),\n"
-    "                           sent (folder name, ex: Sent),\n"
-    "                           smtp_host (ex: smtp.example.com),\n"
-    "                           smtp_port (ex: 465 or 587),\n"
-    "                           trash (folder name, ex: Trash),\n"
-    "                           user (ex: example@example.com or example).\n"
-    "   ~/.nmail/ui.conf        customizes UI settings\n"
-    "   ~/.nmail/secret.conf    stores saved passwords\n"
+    "   ~/.config/nmail/auth.conf  configures custom oauth2 client id and secret\n"
+    "   ~/.config/nmail/main.conf  configures mail account and general settings,\n"
+    "                              for full functionality the following fields\n"
+    "                              shall be configured:\n"
+    "                              address (ex: example@example.com),\n"
+    "                              drafts (folder name, ex: Drafts),\n"
+    "                              imap_host (ex: imap.example.com),\n"
+    "                              imap_port (ex: 993),\n"
+    "                              inbox (folder name, ex: Inbox),\n"
+    "                              name (ex: Firstname Lastname),\n"
+    "                              sent (folder name, ex: Sent),\n"
+    "                              smtp_host (ex: smtp.example.com),\n"
+    "                              smtp_port (ex: 465 or 587),\n"
+    "                              trash (folder name, ex: Trash),\n"
+    "                              user (ex: example@example.com or example).\n"
+    "   ~/.config/nmail/ui.conf    customizes UI settings\n"
     "\n"
     "Report bugs at https://github.com/d99kris/nmail\n"
     "\n";
@@ -511,7 +515,7 @@ static void ShowHelp()
 static void ShowVersion()
 {
   std::cout <<
-    Version::GetUiAppVersion() << "\n"
+    Version::GetAppName(true /*p_WithVersion*/) << "\n"
     "\n"
     "Copyright (c) 2019-2024 Kristofer Berggren\n"
     "\n"
