@@ -75,6 +75,7 @@ std::string Util::m_LocalizedSubjectPrefixes;
 static std::map<std::string, int> s_KeyCodes =
 {
   // additional keys
+  { "KEY_NONE", KEY_NONE },
   { "KEY_TAB", KEY_TAB },
   { "KEY_RETURN", KEY_RETURN },
   { "KEY_SPACE", KEY_SPACE },
@@ -535,14 +536,18 @@ std::string Util::GetDefaultHtmlToTextConvertCmd()
   std::string result;
   const std::string& commandOutPath = Util::GetTempFilename(".txt");
   const std::string& command =
-    std::string("which pandoc w3m lynx elinks 2> /dev/null | head -1 > ") + commandOutPath;
+    std::string("which html2nmail pandoc w3m lynx elinks 2> /dev/null | head -1 > ") + commandOutPath;
   if (system(command.c_str()) == 0)
   {
     std::string output = Util::ReadFile(commandOutPath);
     output.erase(std::remove(output.begin(), output.end(), '\n'), output.end());
     if (!output.empty())
     {
-      if (output.find("/pandoc") != std::string::npos)
+      if (output.find("/html2nmail") != std::string::npos)
+      {
+        result = "html2nmail";
+      }
+      else if (output.find("/pandoc") != std::string::npos)
       {
         result = "pandoc -f html -t plain+literate_haskell --wrap=preserve";
       }
@@ -552,7 +557,7 @@ std::string Util::GetDefaultHtmlToTextConvertCmd()
       }
       else if (output.find("/lynx") != std::string::npos)
       {
-        result = "lynx -assume_charset=utf-8 -display_charset=utf-8 -nomargins -dump -stdin";
+        result = "lynx -assume_charset=utf-8 -display_charset=utf-8 -nomargins -dump";
       }
       else if (output.find("/elinks") != std::string::npos)
       {
@@ -1953,6 +1958,24 @@ std::string Util::ZeroPad(uint32_t p_Num, int32_t p_Len)
   return str;
 }
 
+bool Util::GetQuotePrefix(const std::string& p_String, std::string& p_Prefix, std::string& p_Line)
+{
+  std::smatch sm;
+  std::regex re("^(( *> *)+)");
+  if (std::regex_search(p_String, sm, re))
+  {
+    p_Prefix = sm.str();
+    p_Line = sm.suffix();
+    return true;
+  }
+  else
+  {
+    p_Prefix.clear();
+    p_Line = p_String;
+    return false;
+  }
+}
+
 bool Util::GetQuotePrefix(const std::wstring& p_String, std::wstring& p_Prefix, std::wstring& p_Line)
 {
   std::wsmatch sm;
@@ -2159,7 +2182,7 @@ void Util::JumpToPrevMatch(std::wstring& p_Str, int& p_Pos, int p_Offs, std::wst
 
 void Util::RemoveNonAlphaNumSpace(std::string& p_Str)
 {
-  auto it = std::remove_if(p_Str.begin(), p_Str.end(), [](char const &c) {
+  auto it = std::remove_if(p_Str.begin(), p_Str.end(), [](char const& c) {
     return !std::isalnum(c) && (c != ' ');
   });
 
@@ -2290,3 +2313,86 @@ bool Util::IsIpAddress(const std::string& p_Str)
 
   return rv;
 }
+
+std::string Util::GetDefaultApplicationDir()
+{
+  char* homeEnv = getenv("HOME");
+  static const std::string homeDir = std::string((homeEnv != nullptr) ? homeEnv : "");
+
+  // Use old ~/.nmail if present, for backward compatibility
+  static const std::string legacyApplicationDir = homeDir + "/.nmail";
+  if (IsDir(legacyApplicationDir))
+  {
+    return legacyApplicationDir;
+  }
+
+  // Common value: export XDG_CONFIG_HOME="$HOME/.config"
+  char* xdgConfigHomeEnv = getenv("XDG_CONFIG_HOME");
+  static const std::string xdgConfigHomeDir = std::string((xdgConfigHomeEnv != nullptr) ? xdgConfigHomeEnv : "");
+  static const std::string configHomeDir = (!xdgConfigHomeDir.empty() ? xdgConfigHomeDir : (homeDir + "/.config"));
+
+  // Typically: ~/.config/nmail
+  static const std::string applicationDir = configHomeDir + "/nmail";
+  return applicationDir;
+}
+
+std::string Util::UnwrapQuotedLines(const std::string& p_Msg, int p_LineLen)
+{
+  const std::vector<std::string> lines = Split(Strip(p_Msg, '\r'), '\n');
+  const int lineCnt = lines.size();
+  std::vector<std::string> outlines;
+  std::string outline;
+  std::string outprefix;
+  std::string curline;
+  std::string curprefix;
+  for (int i = 0; i < lineCnt; ++i)
+  {
+    GetQuotePrefix(lines[i], curprefix, curline);
+    const std::string curtrimline = Trim(curline);
+    const int curfirstwordlen = curtrimline.empty() ? 0 : Split(curtrimline, ' ').at(0).size();
+    const int prevlinesize = (i > 0) ? lines[i - 1].size() : 0;
+    if ((outprefix == curprefix) && !curtrimline.empty() && ((prevlinesize + curfirstwordlen + 1) >= p_LineLen))
+    {
+      // Same prefix and previous line plus current lines first word exceed wrap width -> unwrap and append to outline
+      if (!outline.empty())
+      {
+        outline += " ";
+      }
+
+      outline += curline;
+    }
+    else
+    {
+      if (!outline.empty())
+      {
+        // Terminate non-empty outline and start new
+        outlines.push_back(outprefix + outline);
+        outline = "";
+      }
+
+      if (curtrimline.empty())
+      {
+        // Append blank line for empty input line
+        outlines.push_back(curprefix + curtrimline);
+        outline = "";
+      }
+      else
+      {
+        // Set outline to current line
+        outline = curtrimline;
+      }
+
+      // Update outprefix
+      outprefix = curprefix;
+    }
+  }
+
+  if (!outline.empty())
+  {
+    // Append any remaining buffered outline
+    outlines.push_back(outprefix + outline);
+  }
+
+  // Return joined string
+  return Join(outlines, "\n");
+}
